//
//	SynchronizationServer.h
//	TicTacToe
//
//	Abstract class
//
//	Created by Ingo Kasprzak on 11.10.12.
//	Copyright (c) 2012 Silutions. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SynchronizationServerDelegate.h"

/*
 * COMMANDS
 */

#define SyncDatabaseSyncIdentificationCommand @"identify"
// data: Unique device id string from managed object [Device uid]

#define SyncDatabaseSyncSynchronizeCommand @"synchronize"
// data: Set with property dictionaries

#define SyncDatabaseSyncSynchronizeBidirectionalCommand @"synchronizeBidirectional"
// data: Set with property dictionaries

#define SyncDatabaseSyncReceiptCommand @"receipt"
// data: command peerID to confirm


@interface SynchronizationServer : NSObject
@property (weak) id<SynchronizationServerDelegate> delegate;

- (void)addConnectedDevice:(NSString*)deviceUID withPeerIdentifier:(NSString*)peerID;
- (void)removePeerIdentifier:(NSString*)peerID;
- (NSArray*)peerIdentifiersForDeviceUIDs:(NSArray*)deviceUIDs;
- (NSString*)deviceUIDForPeerID:(NSString*)peerID;

- (NSArray*)connectedDeviceUIDs;

- (BOOL)sendObject:(id)object withCommand:(NSString*)command toDevices:(NSArray*)deviceUIDs wantsReceipt:(BOOL)yesOrNo;
- (BOOL)sendObject:(id)object withCommand:(NSString*)command toPeers:(NSArray*)peerIDs wantsReceipt:(BOOL)yesOrNo;

- (void)receivedModifiedObjects:(NSSet*)objects;
- (void)receivedBidirectionalRequestAndModifiedObjects:(NSSet*)objects;
- (void)receivedReceiptForCommand:(NSString*)ucid fromPeer:(NSString*)peer;
- (void)receivedIdentification:(NSString*)deviceUID fromPeer:(NSString*)peer;
- (void)receivedDisconnection:(NSString*)deviceUID fromPeer:(NSString*)peer;

// Die folgenden Methoden müssen von der nächsthöheren Subklasse implementiert werden!
- (BOOL) startServer;
- (BOOL) stopServer;
- (BOOL) sendData:(NSData*)data toDevices:(NSArray*)deviceUIDs;
- (BOOL) sendData:(NSData*)data toPeers:(NSArray*)peerIDs;
- (BOOL) receivedData:(NSData*)data fromPeer:(NSString*)peer;


@end
